type t;;

external of_int32s : int32 * int32 * int32 * int32 * int32 * int32 * int32 * int32 -> t = "c_of_int32s";;
external to_int32s : t -> int32 * int32 * int32 * int32 * int32 * int32 * int32 * int32 = "c_to_int32s";;
external of_small_int : int -> t = "c_of_small_int";; (* must be smaller than 0x7FFF *)

external modinv_n : t -> t = "c_inv_n";;
external modinv_p : t -> t = "c_inv_p";;
external sqrt_p : t -> t = "c_sqrt_p";;

external add_p : t -> t -> t = "c_add_p";;
external sub_p : t -> t -> t = "c_sub_p";;
external mul_p : t -> t -> t = "c_mul_p";;
external neg_p : t -> t = "c_neg_p";;
external shr : t -> int -> t = "c_shr";;

external eq : t -> t -> bool = "c_eq";;
external cmp : t -> t -> int = "c_cmp";; (* Returns -1, 0, +1 depending on the comparison *)
let gt x y = (cmp x y = 1);;
external isodd : t -> bool = "c_isodd";;
external iszero : t -> bool = "c_iszero";;

let iseven t = not (isodd t);;

(* Completely little endian: lowest bits is in front of the 256 bit sequence, only useful for Z.*_strings *)
external of_lebits : string -> t = "c_of_lebits";;
external to_lebits : t -> string = "c_to_lebits";;
let of_string s = of_lebits (Z.to_bits (Z.of_string s));;
let to_string x = Z.to_string (Z.of_bits (to_lebits x));;

external of_bebits : string -> t = "c_of_bebits";;
external to_bebits : t -> string = "c_to_bebits";;

type ge;;
external curve_y : bool -> t -> ge = "c_curve_y";; (* TODO: maybe we want to return t immediately? *)
external ge_of_xy : t -> t -> ge = "c_ge_of_xy";;
external ge_to_xyo : ge -> (t * t) option = "c_ge_to_xyo";;
external smulp : t -> ge -> ge = "c_smulp";;
external add_pt : ge -> ge -> ge = "c_add_pt";;

external sha256 : string -> Be256.t = "c_sha256_bebits";;
external sha256_round :
  (int32*int32*int32*int32*int32*int32*int32*int32) ->
  (int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32) ->
  (int32*int32*int32*int32*int32*int32*int32*int32) = "c_sha256_round";;

(* given the arrays currhashval currblock updates the former *)
let sha256_round_arrays chv cb =
  let a1 = chv.(0),chv.(1),chv.(2),chv.(3),chv.(4),chv.(5),chv.(6),chv.(7) in
  let a2 = cb.(0),cb.(1),cb.(2),cb.(3),cb.(4),cb.(5),cb.(6),cb.(7),
           cb.(8),cb.(9),cb.(10),cb.(11),cb.(12),cb.(13),cb.(14),cb.(15) in
  let (a,b,c,d,e,f,g,h) = sha256_round a1 a2 in
  chv.(0) <- a; chv.(1) <- b; chv.(2) <- c; chv.(3) <- d;
  chv.(4) <- e; chv.(5) <- f; chv.(6) <- g; chv.(7) <- h
;;

(* Should be removed as soon as not needed *)
let sha256_int32p8 x = Be256.to_int32p8 (sha256 x);;

external ripemd160 : Be256.t -> Be160.t = "c_ripemd160_be_be";;

(* Should be removed as soon as not needed *)
let ripemd160_int32s :
  (int32*int32*int32*int32*int32*int32*int32*int32) ->
  (int32*int32*int32*int32*int32) =
  fun x -> Be160.to_int32p5 (ripemd160 (Be256.of_int32p8 x));;

(*  These also work, but no need to expose them
external ripemd160_round :
  (int32*int32*int32*int32*int32) ->
  (int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32*int32) ->
  (int32*int32*int32*int32*int32) = "c_ripemd160_round";;

let ripemd160_round_arrays chv cb =
  let a1 = chv.(0),chv.(1),chv.(2),chv.(3),chv.(4) in
  let a2 = cb.(0),cb.(1),cb.(2),cb.(3),cb.(4),cb.(5),cb.(6),cb.(7),
           cb.(8),cb.(9),cb.(10),cb.(11),cb.(12),cb.(13),cb.(14),cb.(15) in
  let (a,b,c,d,e) = ripemd160_round a1 a2 in
  chv.(0) <- a; chv.(1) <- b; chv.(2) <- c; chv.(3) <- d; chv.(4) <- e;;
 *)
